#ifndef _VELIB_QT_VE_QITEM_TABLE_MODEL_HPP_
#define _VELIB_QT_VE_QITEM_TABLE_MODEL_HPP_

#include <QAbstractItemModel>
#include <QHash>
#include <QModelIndex>
#include <QVariant>

#include <velib/qt/ve_qitem.hpp>

class VeQItemTableModel : public QAbstractItemModel
{
	Q_OBJECT

public:
	enum VeQItemTableRoles {
		UniqueIdRole = Qt::UserRole + 1,
		IdRole,
		ValueRole,
		StateRole,
		TextRole,
		TextStateRole,
	};

	enum VeQItemModelOptions {
		VeQItemModelNoOptions				= 0,
		VeQItemModelDontAddItem				= 0x01,
		VeQItemModelAddChildren				= 0x02,
		VeQItemModelAddAllChildren			= 0x04,
		VeQItemModelAddNoneLeaves			= 0x08,
		VeQItemModelWithText				= 0x1000,
		VeQItemModelWithSettingInfo			= 0x2000,
	};

	VeQItemTableModel(int flags = VeQItemModelAddAllChildren, QObject *parent = 0);

	void addItem(VeQItem *item);
	QVariant data(const QModelIndex &index, int role) const;
	Qt::ItemFlags flags(const QModelIndex &index) const;
	bool setData(const QModelIndex & index, const QVariant &value, int role = Qt::EditRole);
	QVariant headerData(int section, Qt::Orientation orientation,
						int role = Qt::DisplayRole) const;
	QModelIndex index(int row, int column,
					  const QModelIndex &parent = QModelIndex()) const;
	QModelIndex parent(const QModelIndex &index) const;
	int rowCount(const QModelIndex &parent = QModelIndex()) const;
	int columnCount(const QModelIndex &parent = QModelIndex()) const;
	QHash<int, QByteArray> roleNames() const;

private slots:
	void onChildAdded(VeQItem *item);
	void onRecursiveChildAdded(VeQItem *item);
	void onItemAboutToBeRemoved(VeQItem *item);
	void onValueChanged(VeQItem *item);
	void onStateChanged(VeQItem *item);
	void onTextChanged(VeQItem *item);
	void onTextStateChanged(VeQItem *item);
	void onDynamicPropertyChanged(VeQItem *item, const char *name);
	void addExistingChildren(VeQItem *item, void *ctx);

private:
	void cellChanged(VeQItem *item, QString column);
	void setupValueChanges(VeQItem *item, VeQItemModelOptions options = VeQItemModelNoOptions);

	QList<QString> mColumns;
	QMap<QString, VeQItem *> mMap;
	int mFlags;
};

#endif
