#ifndef _VELIB_QT_VE_QITEMS_DBUS_HPP_
#define _VELIB_QT_VE_QITEMS_DBUS_HPP_

#include <QHash>
#include <QtDBus>
#include <QDBusServiceWatcher>
#include <velib/qt/ve_qitem.hpp>

class VBusItemProxy;
class VeQItemDbusProducer;

// Implementation of a VeQItem over the dbus
class VeQItemDbus : public VeQItem, protected QDBusContext
{
	Q_OBJECT

public:
	VeQItemDbus(VeQItemDbusProducer *producer);
	bool introspect();
	virtual QVariant getValue() { return getValue(false); }
	virtual QString getText() { return getText(false); }
	virtual QVariant itemProperty(const char *name) { return itemProperty(name, false); }
	virtual int setValue(QVariant const &value);
	VeQItemDbusProducer *producer();
	void serviceRegistrationChanged(bool registered);

protected:
	virtual void setParent(QObject *parent);

private slots:
	void valueObtained(QDBusPendingCallWatcher *call);
	void textObtained(QDBusPendingCallWatcher *call);
	void introspectObtained(QDBusPendingCallWatcher *call);
	void onPropertiesChanged(const QVariantMap &changes);
	void setValueDone(QDBusPendingCallWatcher *call);

	void maxObtained(QDBusPendingCallWatcher *call);
	void minObtained(QDBusPendingCallWatcher *call);
	void defaultObtained(QDBusPendingCallWatcher *call);

private:
	QVariant getValue(bool force);
	QString getText(bool force);
	QVariant itemProperty(const char *name, bool force);

	void propertyObtained(const char *name, QDBusPendingCallWatcher *call);
	void demarshallVariantForQml(QVariant &variant, QString &signature);

	VBusItemProxy *mInterface;
	QString mSignature;
	QVariant mPendingSetValue;

	bool mRequestValueWhenOnline;
	bool mRequestTextWhenOnline;
	bool mRequestDefaultWhenOnline;
	bool mRequestMaxWhenOnline;
	bool mRequestMinWhenOnline;
};

class VeQItemDbusProducer : public VeQItemProducer, protected QDBusContext
{
	Q_OBJECT

public:
	VeQItemDbusProducer(VeQItem *root, QString id, bool findVictronServices = true,
						bool bulkInitOfNewService = true, QObject *parent = 0);
	~VeQItemDbusProducer();

	virtual bool open(const QString &address = "session", const QString &qtDbusName = "qtdbus");
	virtual bool open(const QDBusConnection &dbusConnection);

	virtual VeQItem *createItem();
	QDBusConnection &dbusConnection() { return mDbus; }
	void serviceWatchAdd(QString serviceName);

	bool getListenIndividually() const { return mListenIndividually; }

	void setListenIndividually(bool l);

private slots:
	void onServiceOwnerChanged(const QString &serviceName, const QString &oldOwner, const QString &newOwner);

private slots:
	void onServiceRegistered(const QString &service);
	void onServiceUnregistered(const QString &service);

public:
	bool isServiceRegistered(QString serviceName);
	bool getBulkInit() { return mBulkInitOfNewService; }

private:
	void onServiceRegistrationChanged(const QString &serviceName, bool registered);
	void disconnectChildren(VeQItem *root, VeQItem *item);
	QHash<QString, bool> mServiceRegistered;
	QHash<QString, QDBusServiceWatcher *> mServiceWatchers;
	virtual bool open() { return false; }

private:
	QDBusConnection mDbus;
	bool mFindVictronServices;
	bool mBulkInitOfNewService;
	bool mListenIndividually;
};

#endif
