#ifndef VEBUSVIRTUALOBJECT_H
#define VEBUSVIRTUALOBJECT_H

#include <QDBusConnection>
#include <QDBusVirtualObject>
#include <QList>
#include <QVariantMap>

class VeQItem;

/// Represents a single D-Bus service and handles all method calls.
/// This is used by VeQItemDbusPublisher to publish VeQItem tree on the D-Bus.
class VeQItemDbusVirtualObject : public QDBusVirtualObject
{
	Q_OBJECT
public:
	VeQItemDbusVirtualObject(const QDBusConnection &connection, VeQItem *root, QObject *parent = 0);

	QString serviceName() const;

	VeQItem *root();

	bool registerService();

	bool unregisterService();

	virtual QString introspect(const QString &path) const;

	virtual bool handleMessage(const QDBusMessage &message, const QDBusConnection &connection);

private slots:
	void onChildAdded(VeQItem *item);

	void onChildRemoved(VeQItem *item);

	void onValueChanged(VeQItem *item);

	void onTextChanged(VeQItem *item);

private:
	bool handleGetValue(const QDBusMessage &message, const QDBusConnection &connection,
						VeQItem *item);

	bool handleGetText(const QDBusMessage &message, const QDBusConnection &connection,
					   VeQItem *item);

	bool handleSetValue(const QDBusMessage &message, const QDBusConnection &connection,
						VeQItem *producer);

	void buildTree(VeQItem *root, VeQItem *item, QVariantMap &map, bool useText = false);

	void addPending(VeQItem *item);

	Q_INVOKABLE void processPending();

	void sendPropertiesChanged(VeQItem *item);

	void connectItem(VeQItem *item);

	void disconnectItem(VeQItem *item);

	VeQItem *findItem(const QString &path) const;

	static void normalizeVariant(QVariant &v);

	static void normalizeMap(QVariantMap &map);

	static void normalizeList(QVariantList &list);

	static void denormalizeVariant(QVariant &v);

	QDBusConnection mConnection;
	VeQItem *mRoot;
	QList<VeQItem *> mPendingChanges;
};

#endif // VEBUSVIRTUALOBJECT_H
